import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Infiltration


class TestInfiltration(VariableUnitTest):

    def test_Infiltration_ground_truth(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/Infiltration.npy"),
            Infiltration.Infiltration(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                      z.CNI_0, z.AntMoist_0,
                                      z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN), decimal=7)

    # @skip('Not Ready Yet.')
    def test_Infiltration(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Infiltration.Infiltration_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                        z.CNI_0, z.AntMoist_0,
                                        z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN),
            Infiltration.Infiltration(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                      z.CNI_0, z.AntMoist_0,
                                      z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN), decimal=7)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Imported from StreamBank.bas
"""

import logging
from .Input.LandUse.AreaTotal import AreaTotal_f
from .Input.LandUse.Ag.AGSTRM import AGSTRM_f
from .Input.LandUse.Ag.TileDrain import TileDrain_f
from .Output.AvAnimalNSum.AnimalN import AnimalN_f
from .Output.Loading.StreamBankEros import StreamBankEros_f
from .Output.Loading.StreamBankEros_1 import StreamBankEros_1_f
from .Output.Loading.StreamBankN_1 import StreamBankN_1_f

log = logging.getLogger(__name__)


def CalculateStreamBankEros(z, Y):
    # CALCULATE THE STREAM BANK SEDIMENT AND N AND P
    for i in range(12):
        # CALCULATE ER FACTOR FOR STREAMBANK EROSION

        z.StreamBankP[Y][i] = \
        StreamBankEros_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                         z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                         z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                         z.RecessionCoef, z.SeepCoef,
                         z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow,
                         z.StreamWithdrawal,
                         z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                         z.AvSlope,
                         z.SedAAdjust, z.StreamLength)[Y][i] * (z.SedPhos / 1000000) * z.BankPFrac
        # CALCULATIONS FOR STREAM BANK STABILIZATION AND FENCING
        z.PURBBANK = 0
        z.FCURBBANK = 0

        z.PSTAB = 0
        z.PURBBANK = 0
        if z.n42b > 0:
            z.PSTAB = (z.n46c / z.n42b) * z.StreamBankP[Y][i] * z.n77c
            z.PURBBANK = (z.UrbBankStab / z.n42b) * z.StreamBankP[Y][i] * z.n77c

        z.PFEN = 0
        if z.n42 > 0:
            z.PFEN = (z.n45 / z.n42) * z.StreamBankP[Y][i] * AGSTRM_f(z.AgLength, z.StreamLength) * z.n77

        z.StreamBankP[Y][i] = z.StreamBankP[Y][i] - (z.PSTAB + z.PFEN + z.PURBBANK)
        if z.StreamBankP[Y][i] < 0:
            z.StreamBankP[Y][i] = 0

        # CALCULATE ANNUAL STREAMBANK N AND P AND SEDIMENT
        z.StreamBankPSum[Y] += z.StreamBankP[Y][i]
        z.StreamBankErosSum[Y] += \
            StreamBankEros_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                               z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0,
                               z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef
                               , z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow,
                               z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                               z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d,
                               z.AgLength, z.n42, z.n45, z.n85, z.UrbBankStab)[Y][i]

        # GROUNDWATER N LOADS ARE REDUCED BASED ON SPECIFIC BMPS
        z.GWNRF = 0
        z.CHNGN1 = 0
        z.CHNGN2 = 0
        z.CHNGN3 = 0
        z.CHNGN4 = 0
        z.CHNGN5 = 0
        z.CHNGNTOT = 0
        z.PCTN1 = 0
        z.PCTN2 = 0
        z.PCTN3 = 0
        z.PCTN4 = 0
        z.PCBMPAC = 0
        z.HPBMPAC = 0
        z.BMPACRES = 0
        z.PCTAG = 0
        z.RCNMAC = 0
        z.HPNMAC = 0

        z.CHNGN1 = z.n25 / 100
        z.CHNGN2 = z.n26 / 100
        z.CHNGN3 = z.n27 / 100
        z.CHNGN4 = z.n27b / 100
        z.CHNGN5 = z.n28 / 100
        z.CHNGNTOT = z.CHNGN1 + z.CHNGN2 + z.CHNGN3 + z.CHNGN4 + z.CHNGN5

        if AreaTotal_f(z.Area) > 0 and z.n23 > 0 and z.n42 > 0 and z.n42b > 0:
            z.PCTAG = (z.n23 + z.n24) / AreaTotal_f(z.Area)
            z.GroundNitr[Y][i] -= z.GroundNitr[Y][i] * ((z.n28b / 100) * z.n23) / z.n23 * z.PCTAG * z.n70
            z.GroundNitr[Y][i] -= z.GroundNitr[Y][i] * (z.n43 / z.n42) * (z.n42 / z.n42b) * z.PCTAG * z.n64
            z.GroundNitr[Y][i] -= (z.GroundNitr[Y][i] * (
                    (((z.n29 / 100) * z.n23) + ((z.n37 / 100) * z.n24)) / (z.n23 + z.n24))) * z.PCTAG * z.n68

        # Groundwater P loads are reduced based on extent of nutrient management BMP
        z.RCNMAC = (z.n28b / 100) * z.n23
        z.HPNMAC = (z.n35b / 100) * z.n24

        if AreaTotal_f(z.Area) > 0:
            z.GroundPhos[Y][i] -= (((z.RCNMAC + z.HPNMAC) / AreaTotal_f(z.Area)) *
                                   z.GroundPhos[Y][i] * z.n78)

        z.GroundNitrSum[Y] += z.GroundNitr[Y][i]
        z.GroundPhosSum[Y] += z.GroundPhos[Y][i]

        z.TileDrainSum[Y] += \
            TileDrain_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                        z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                        z.RecessionCoef, z.SeepCoef, z.Landuse,
                        z.TileDrainDensity)[Y][i]
        z.TileDrainNSum[Y] += z.TileDrainN[Y][i]
        z.TileDrainPSum[Y] += z.TileDrainP[Y][i]
        z.TileDrainSedSum[Y] += z.TileDrainSed[Y][i]
        z.AnimalPSum[Y] += z.AnimalP[Y][i]
        z.AnimalFCSum[Y] += z.AnimalFC[Y][i]
        z.WWOrgsSum[Y] += z.WWOrgs[Y][i]
        z.SSOrgsSum[Y] += z.SSOrgs[Y][i]
        z.UrbOrgsSum[Y] += z.UrbOrgs[Y][i]
        z.TotalOrgsSum[Y] += z.TotalOrgs[Y][i]
        z.WildOrgsSum[Y] += z.WildOrgs[Y][i]

        z.GRLostBarnPSum[Y] += z.GRLostBarnP[Y][i]
        z.GRLostBarnFCSum[Y] += z.GRLostBarnFC[Y][i]
        z.NGLostBarnPSum[Y] += z.NGLostBarnP[Y][i]
        z.NGLostBarnFCSum[Y] += z.NGLostBarnFC[Y][i]
        z.NGLostManPSum[Y] += z.NGLostManP[Y][i]

        z.TotNitr[Y][i] += StreamBankN_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                           z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                           z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                           z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                           z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal, z.GroundWithdrawal,
                                           z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                                           z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.AgLength,
                                           z.UrbBankStab, z.SedNitr, z.BankNFrac, z.n69c, z.n45, z.n69, z.n46c, z.n42)[
                               Y][i] + \
                           z.TileDrainN[Y][i] + \
                           AnimalN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                     z.AnimalDailyN, z.NGAppNRate, z.Prec, z.DaysMonth,
                                     z.NGPctSoilIncRate, z.GRPctManApp, z.GRAppNRate, z.GRPctSoilIncRate, z.NGBarnNRate,
                                     z.AWMSNgPct, z.NgAWMSCoeffN,
                                     z.RunContPct, z.RunConCoeffN, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct,
                                     z.GrAWMSCoeffN, z.PctStreams, z.GrazingNRate)[Y][i]
        z.TotPhos[Y][i] += z.StreamBankP[Y][i] + z.TileDrainP[Y][i] + z.AnimalP[Y][i]

        z.TotNitrSum[Y] += StreamBankN_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                           z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                           z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                           z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                           z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal, z.GroundWithdrawal,
                                           z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                                           z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.AgLength,
                                           z.UrbBankStab, z.SedNitr, z.BankNFrac, z.n69c, z.n45, z.n69, z.n46c, z.n42)[
                               Y][i] + \
                           z.TileDrainN[Y][i] + \
                           AnimalN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                     z.AnimalDailyN, z.NGAppNRate, z.Prec, z.DaysMonth,
                                     z.NGPctSoilIncRate, z.GRPctManApp, z.GRAppNRate, z.GRPctSoilIncRate, z.NGBarnNRate,
                                     z.AWMSNgPct, z.NgAWMSCoeffN,
                                     z.RunContPct, z.RunConCoeffN, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct,
                                     z.GrAWMSCoeffN, z.PctStreams, z.GrazingNRate)[Y][i]
        z.TotPhosSum[Y] += z.StreamBankP[Y][i] + z.TileDrainP[Y][i] + z.AnimalP[Y][i]



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.AgAnimal import NAGBUFFER


class TestNAGBUFFER(VariableUnitTest):

    def test_NAGBUFFER(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NAGBUFFER.NAGBUFFER_f(z.n42, z.n43, z.n64, z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals,
                                  z.AvgAnimalWt, z.AnimalDailyN, z.NGBarnNRate,
                                  z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN,
                                  z.GRPctManApp, z.PctGrazing, z.GRBarnNRate,
                                  z.AWMSGrPct, z.GrAWMSCoeffN, z.PctStreams, z.NGAppNRate, z.NGPctSoilIncRate,
                                  z.GRAppNRate, z.GRPctSoilIncRate,
                                  z.GrazingNRate),
            NAGBUFFER.NAGBUFFER(z.n42, z.n43, z.n64, z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals,
                                z.AvgAnimalWt, z.AnimalDailyN, z.NGBarnNRate,
                                z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN,
                                z.GRPctManApp, z.PctGrazing, z.GRBarnNRate,
                                z.AWMSGrPct, z.GrAWMSCoeffN, z.PctStreams, z.NGAppNRate, z.NGPctSoilIncRate,
                                z.GRAppNRate, z.GRPctSoilIncRate,
                                z.GrazingNRate), decimal=7)



from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.Ag.TileDrain import TileDrain
from gwlfe.Input.LandUse.Ag.TileDrain import TileDrain_f
from gwlfe.Memoization import memoize


def AvTileDrain(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                Landuse, TileDrainDensity):
    result = zeros((12,))
    tile_drain = TileDrain(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                           Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                           RecessionCoef, SeepCoef, Landuse, TileDrainDensity)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += tile_drain[Y][i] / NYrs
    return result


@memoize
def AvTileDrain_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                  Landuse, TileDrainDensity):
    return sum(
        TileDrain_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                    Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                    RecessionCoef, SeepCoef, Landuse, TileDrainDensity), axis=0)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import nRunoff


class TestnRunoff(VariableUnitTest):
    def test_nRunoff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            nRunoff.nRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                              z.Grow_0, z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth,
                              z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2),
            nRunoff.nRunoff(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                            z.Grow_0, z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth,
                            z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.AvAnimalNSum import AnimalN


class TestAnimalN(VariableUnitTest):
    def test_AnimalN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AnimalN.AnimalN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                              z.NGAppNRate, z.Prec, z.DaysMonth,
                              z.NGPctSoilIncRate, z.GRPctManApp, z.GRAppNRate, z.GRPctSoilIncRate, z.NGBarnNRate,
                              z.AWMSNgPct, z.NgAWMSCoeffN,
                              z.RunContPct, z.RunConCoeffN, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct, z.GrAWMSCoeffN,
                              z.PctStreams, z.GrazingNRate),
            AnimalN.AnimalN(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                            z.NGAppNRate, z.Prec, z.DaysMonth,
                            z.NGPctSoilIncRate, z.GRPctManApp, z.GRAppNRate, z.GRPctSoilIncRate, z.NGBarnNRate,
                            z.AWMSNgPct, z.NgAWMSCoeffN,
                            z.RunContPct, z.RunConCoeffN, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct, z.GrAWMSCoeffN,
                            z.PctStreams, z.GrazingNRate), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Precipitation


class TestPrecipitation(VariableUnitTest):

    def test_Precipitation(self):
        z = self.z
        temp_f = Precipitation.Precipitation_f(z.Prec)
        temp = Precipitation.Precipitation(z.NYrs, z.DaysMonth, z.Prec)
        np.testing.assert_array_almost_equal(Precipitation.Precipitation_f(z.Prec),
                                             Precipitation.Precipitation(z.NYrs, z.DaysMonth, z.Prec), decimal=7)



from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.InitSnow import InitSnow
from gwlfe.Input.WaterBudget.InitSnowYesterday import InitSnowYesterday
from gwlfe.Input.WaterBudget.Melt import Melt, Melt_f
from gwlfe.Memoization import memoize


@memoize
def Melt_1(NYrs, DaysMonth, InitSnow_0, Temp, Prec):
    result = zeros((NYrs, 12, 31))
    init_snow = InitSnow(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    init_snow_yesterday = InitSnow_0
    melt = Melt(NYrs, DaysMonth, Temp, InitSnow_0, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and init_snow_yesterday > 0.001 and melt[Y][i][j] > init_snow_yesterday:
                    result[Y][i][j] = init_snow_yesterday
                else:
                    result[Y][i][j] = melt[Y][i][j]
                init_snow_yesterday = init_snow[Y][i][j]
    return result


@memoize
def Melt_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec):
    # result = np.zeros((NYrs, 12, 31))
    init_snow_yesterday = InitSnowYesterday(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    melt = Melt_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec)
    melt[where((Temp > 0) & (init_snow_yesterday > 0.001) & (melt > init_snow_yesterday))] = init_snow_yesterday[
        where((Temp > 0) & (init_snow_yesterday > 0.001) & (melt > init_snow_yesterday))]
    return melt



from numpy import zeros

from .N7b import N7b
from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN_2 import GRLBN_2
from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import AvGRStreamN
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnNSum import NGLostBarnNSum
from gwlfe.BMPs.AgAnimal.NAWMSL_2 import NAWMSL_2
from gwlfe.BMPs.AgAnimal.NAWMSP_2 import NAWMSP_2
from gwlfe.BMPs.AgAnimal.NFENCING import NFENCING
from gwlfe.BMPs.AgAnimal.NRUNCON_2 import NRUNCON_2


def N7b_2(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
          NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct, NgAWMSCoeffN,
          RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate, n41b,
          n85h, n41d, n85j, n41f, n85l, n42, n45, n69, n43, n64):
    result = zeros((NYrs))
    n7b = N7b(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
              GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
              GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN,
              n41d,
              n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64)
    nawmsl = NAWMSL_2(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                      GRBarnNRate,
                      Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h)
    nawmsp = NAWMSP_2(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                      Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j)
    nruncon = NRUNCON_2(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                        GRBarnNRate,
                        Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate,
                        AWMSNgPct,
                        NgAWMSCoeffN, n41f, n85l)
    nfencing = NFENCING(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, n42, n45, n69)
    nglbn = NGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                           Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    grlbn = GRLBN_2(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                    Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    grsn = AvGRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    n7b_carryover = n7b
    for y in range(NYrs):
        nagbuffer_2 = (n43 / n42) * n64 * (n7b_carryover - (nglbn[y] + grlbn[y] + grsn))
        result[y] = n7b_carryover - (nawmsl[y] + nawmsp[y] + nruncon[y] + nfencing + nagbuffer_2)
        n7b_carryover = result[y]
    return result


def N7b_2_f():
    pass



from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN_f
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import AvNGLostBarnNSum
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import AvNGLostBarnNSum_f


def NRUNCON(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
            Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate, AWMSNgPct,
            NgAWMSCoeffN, n41f, n85l):
    grlbn = GRLBN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                  Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    nglbn = AvNGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                             Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    result = (n41f / 100) * n85l * (grlbn + nglbn)
    return result


def NRUNCON_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
              Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate, AWMSNgPct,
              NgAWMSCoeffN, n41f, n85l):
    grlbn = GRLBN_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                    Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    nglbn = AvNGLostBarnNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                               Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    return (n41f / 100) * n85l * (grlbn + nglbn)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Loads import GRLoadN
from gwlfe.enums import YesOrNo


class TestGRLoadN(VariableUnitTest):
    def test_newGRLoadN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRLoadN.GRLoadN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            GRLoadN.GRLoadN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN)[
                z.GrazingAnimal_0 == YesOrNo.YES], decimal=7)



from .VariableUnitTest import VariableUnitTest


class TestAttenN(VariableUnitTest):
    pass

    # @skip("not ready")
    # def test_AttenN(self):
    #     z = self.z
    #     np.testing.assert_array_almost_equal(
    #         AttenN.AttenN_2(),
    #         AttenN.AttenN(), decimal=7)



from numpy import maximum
from numpy import minimum
from numpy import ndarray
from numpy import reshape
from numpy import tile
from numpy import zeros

from gwlfe.AFOS.nonGrazingAnimals.Loads.NGAppManN import NGAppManN
from gwlfe.AFOS.nonGrazingAnimals.Loads.NGAppManN import NGAppManN_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.LossFactAdj import LossFactAdj
from gwlfe.MultiUse_Fxns.LossFactAdj import LossFactAdj_f


def NGLostManN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
               NGPctSoilIncRate):
    # Non-grazing animal losses
    result = zeros((NYrs, 12))
    loss_fact_adj = LossFactAdj(NYrs, Prec, DaysMonth)
    ng_app_man_n = NGAppManN(NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = (ng_app_man_n[i] * NGAppNRate[i] * loss_fact_adj[Y][i]
                            * (1 - NGPctSoilIncRate[i]))
            if result[Y][i] > ng_app_man_n[i]:
                result[Y][i] = ng_app_man_n[i]
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


@memoize
def NGLostManN_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
                 NGPctSoilIncRate):
    lossFactAdj = LossFactAdj_f(Prec, DaysMonth)
    ng_app_man_n = NGAppManN_f(NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    result = tile(ng_app_man_n * NGAppNRate * (1 - NGPctSoilIncRate), NYrs) * ndarray.flatten(lossFactAdj)
    result = minimum(result, tile(ng_app_man_n, NYrs))  # TODO: should eliminate the double tile
    result = maximum(result, 0)
    return reshape(result, (NYrs, 12))



from .AvAnimalNSum import AvAnimalNSum
from gwlfe.BMPs.AgAnimal.NAGBUFFER import NAGBUFFER
from gwlfe.BMPs.AgAnimal.NAGBUFFER import NAGBUFFER_f
from gwlfe.BMPs.AgAnimal.NAWMSL import NAWMSL
from gwlfe.BMPs.AgAnimal.NAWMSL import NAWMSL_f
from gwlfe.BMPs.AgAnimal.NAWMSP import NAWMSP
from gwlfe.BMPs.AgAnimal.NAWMSP import NAWMSP_f
from gwlfe.BMPs.AgAnimal.NFENCING import NFENCING
from gwlfe.BMPs.AgAnimal.NFENCING import NFENCING_f
from gwlfe.BMPs.AgAnimal.NRUNCON import NRUNCON
from gwlfe.BMPs.AgAnimal.NRUNCON import NRUNCON_f
from gwlfe.Output.AvAnimalNSum.AvAnimalNSum import AvAnimalNSum_f


def N7b(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
        GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate,
        Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct,
        NGBarnNRate, NgAWMSCoeffN, n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64):
    av_animal_n_sum = AvAnimalNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                   NGAppNRate,
                                   Prec, DaysMonth,
                                   NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct,
                                   NgAWMSCoeffN,
                                   RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN,
                                   PctStreams, GrazingNRate)
    nawmsl = NAWMSL(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                    Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h)
    nawmsp = NAWMSP(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                    Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j)
    nruncon = NRUNCON(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                      GRBarnNRate,
                      Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate,
                      AWMSNgPct, NgAWMSCoeffN, n41f, n85l)
    nfencing = NFENCING(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, n42, n45, n69)
    nagbuffer = NAGBUFFER(n42, n43, n64, NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                          NGBarnNRate, Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, GRPctManApp,
                          PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, NGAppNRate, NGPctSoilIncRate,
                          GRAppNRate, GRPctSoilIncRate, GrazingNRate)
    result = av_animal_n_sum - (nawmsl + nawmsp + nruncon + nfencing + nagbuffer)
    return result


def N7b_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
          GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate,
          Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct,
          NGBarnNRate, NgAWMSCoeffN, n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64):
    av_animal_n_sum = AvAnimalNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                     NGAppNRate, Prec, DaysMonth, NGPctSoilIncRate, GRPctManApp, GRAppNRate,
                                     GRPctSoilIncRate, NGBarnNRate, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN,
                                     PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate)
    nawmsl = NAWMSL_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                      GRBarnNRate,
                      Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h)
    nawmsp = NAWMSP_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                      Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j)
    nruncon = NRUNCON_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                        GRBarnNRate,
                        Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate,
                        AWMSNgPct, NgAWMSCoeffN, n41f, n85l)
    nfencing = NFENCING_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, n42, n45, n69)
    nagbuffer = NAGBUFFER_f(n42, n43, n64, NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                            NGBarnNRate, Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN,
                            GRPctManApp,
                            PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, NGAppNRate, NGPctSoilIncRate,
                            GRAppNRate, GRPctSoilIncRate, GrazingNRate)
    result = av_animal_n_sum - (nawmsl + nawmsp + nruncon + nfencing + nagbuffer)
    return result



from numpy import repeat
from numpy import tile
from numpy import where
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water, Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.CNum import CNum, CNum_f


@memoize
def Retention(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu))  # Why nlu ?
    c_num = CNum(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, CN, NRur, NUrb, Grow_0)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        if CN[l] > 0:  # TODO:CN is set to zero in datamodel
                            result[Y][i][j][l] = 2540 / c_num[Y][i][j][l] - 25.4
                            if result[Y][i][j][l] < 0:
                                result[Y][i][j][l] = 0
    return result


@memoize
def Retention_f(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu))
    c_num = CNum_f(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, CN, NRur, NUrb, Grow_0)
    cnrur = tile(CN[:NRur][None, None, None, :], (NYrs, 12, 31, 1))
    water = repeat(Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)[:, :, :, None], NRur, axis=3)
    TempE = repeat(Temp[:, :, :, None], NRur, axis=3)
    result[where((TempE > 0) & (water > 0.01) & (cnrur > 0))] = 2540 / c_num[
        where((TempE > 0) & (water > 0.01) & (cnrur > 0))] - 25.4
    result[where(result < 0)] = 0
    return result



import random

import numpy as np
import numpy.ma as ma
# from numba import jit
from numpy import array
from numpy import r_
from numpy import ravel
from numpy import where
from numpy import zeros

leap_year = [False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             True, True, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, True, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, True, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, True, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, True, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False, False, False, False,
             False, False, False, False, False, False, False, False, False, False, False, False]
non_leap_year = [False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, True, True, True, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, True, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, True, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, True, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False, True,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False, False, False, False, False, False, False, False, False, False, False, False,
                 False, False, False]


def mask_builder(DaysMonth):
    ones = ravel(np.ones((12, 31))).astype("int")
    slices = []
    for i, month in enumerate(DaysMonth[0]):
        slices.append(slice(31 * i, 31 * i + month))
    ones[r_[tuple(slices)]] = 0
    return ones


def ymd_to_daily(ymd_array, DaysMonth):
    month_maps = [leap_year if x[1] == 29 else non_leap_year for x in DaysMonth]
    mask = ravel(array(month_maps))
    x = ma.array(ymd_array, mask=mask)
    return x[~x.mask]


def daily_to_ymd(daily_array, NYrs, DaysMonth):
    result = zeros((NYrs * 12 * 31,))
    month_maps = [leap_year if x[1] == 29 else non_leap_year for x in DaysMonth]
    mask = ravel(array(month_maps))
    x = ma.array(result, mask=mask)
    x[~x.mask] = daily_array
    return x.reshape((NYrs, 12, 31))


def ymd_to_daily_slow(ymd_array, NYrs, DaysMonth):
    result = []
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result.append(ymd_array[Y][i][j])
    return array(result)


def get_value_for_yesterday(variable, variable_0, Y_in, i_in, j_in, DaysMonth):
    temp = array(variable)
    key = random.randint(100000000000, 999999999999)  # use something that is hopefully unique so we can find it again
    temp[Y_in, i_in, j_in] = key
    temp = ymd_to_daily(temp, DaysMonth)
    today = where(temp == key)[0]
    if (len(today) != 1):
        raise AssertionError("this array has values equal to the key")

    if (today[0] - 1 >= 0):
        return temp[today[0] - 1]
    else:  # at the first index of the array
        return variable_0



from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.RurQRunoff import RurQRunoff
from gwlfe.MultiUse_Fxns.Runoff.RurQRunoff import RurQRunoff_f
from gwlfe.Output.Loading.NConc import NConc
from gwlfe.Output.Loading.NConc import NConc_f


@memoize
def nRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, NitrConc,
            ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2, LastManureMonth2):
    result = zeros((NYrs, 12, 10))
    rur_q_runoff = RurQRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0)
    n_conc = NConc(NRur, NUrb, NitrConc, ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2,
                   LastManureMonth2)
    for Y in range(NYrs):
        for i in range(12):
            for l in range(NRur):
                result[Y][i][l] = 0.1 * n_conc[i][l] * rur_q_runoff[Y][l][i] * Area[l]
    # += changed to =
    return result


@memoize
def nRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, NitrConc,
              ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2, LastManureMonth2):
    n_conc = NConc_f(NRur, NUrb, NitrConc, ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2,
                     LastManureMonth2)[:, :NRur]

    return 0.1 * n_conc * RurQRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN,
                                       Grow_0) * Area[:NRur]



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.Animals import TotLAEU


class TestTotLAEU(VariableUnitTest):

    def test_TotLAEU(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            TotLAEU.TotLAEU_f(z.NumAnimals, z.AvgAnimalWt),
            TotLAEU.TotLAEU(z.NumAnimals, z.AvgAnimalWt), decimal=7)



from numpy import repeat
from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.Erosiv import Erosiv
from gwlfe.MultiUse_Fxns.Erosion.Erosiv import Erosiv_f


@memoize
def RurEros(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area):
    result = zeros((NYrs, 12, 31, NRur))
    erosiv = Erosiv(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        result[Y][i][j][l] = 1.32 * erosiv[Y][i][j] * KF[l] * LS[l] * C[l] * P[l] * Area[l]
    return result


@memoize
def RurEros_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area):
    erosiv = repeat(Erosiv_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef)[:, :, :, None], NRur, axis=3)
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    water = water[:, :, :, None]
    resized_temp = repeat(Temp[:, :, :, None], NRur, axis=3)
    water_r = repeat(water, NRur, axis=3)
    temp = KF * LS * C * P * Area
    return where((resized_temp > 0) & (water_r > 0.01), 1.32 * erosiv * temp[:NRur], 0.)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import StreamFlowLE


class TestStreamFlowLE(VariableUnitTest):

    def test_StreamFlowLE(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            StreamFlowLE.StreamFlowLE_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                        z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                        z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                        z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                        z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal, z.GroundWithdrawal),
            StreamFlowLE.StreamFlowLE(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                      z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                      z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                      z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                      z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal, z.GroundWithdrawal),
            decimal=7)



from numpy import sum
from numpy import zeros

from gwlfe.Input.WaterBudget.ET_1 import ET_1
from gwlfe.Input.WaterBudget.ET_1 import ET_1_f
from gwlfe.Memoization import memoize


def Evapotrans(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
               ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, ETFlag):
    result = zeros((NYrs, 12))
    et_f = ET_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET,
                DayHrs, MaxWaterCap, ETFlag)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i] = result[Y][i] + et_f[Y][i][j]
    return result


@memoize
def Evapotrans_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                 ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap):
    et_f = ET_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                  AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET,
                  DayHrs, MaxWaterCap)
    return sum(et_f, axis=2)



from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.ErosWashoff import ErosWashoff
from gwlfe.MultiUse_Fxns.Erosion.ErosWashoff import ErosWashoff_f


def LuErosion(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Acoef, KF, LS,
              C, P, Area):
    result = zeros((NYrs, 10))
    eros_washoff = ErosWashoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Acoef, KF, LS,
                               C, P, Area)
    for Y in range(NYrs):
        for i in range(12):
            for l in range(NRur):
                result[Y][l] += eros_washoff[Y][l][i]
    return result


@memoize
def LuErosion_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, Acoef, KF, LS,
                C, P, Area):
    return sum(ErosWashoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, Acoef, KF, LS,
                             C, P, Area), axis=1)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.Animals import GrazingAnimal
from gwlfe.enums import YesOrNo


class TestGrazingAnimal(VariableUnitTest):
    def test_GrazingAnimal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GrazingAnimal.GrazingAnimal_f(z.GrazingAnimal_0),
            GrazingAnimal.GrazingAnimal(z.GrazingAnimal_0) == YesOrNo.YES, decimal=7)



from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN_2 import GRLBN_2
from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN_2 import GRLBN_2_f


def NAWMSL_2(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
             Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h):
    grlbn = GRLBN_2(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                    Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    result = (n41b / 100) * n85h * grlbn
    return result


# def NAWMSL_2_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
#                Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h):
#     return (n41b / 100) * n85h * GRLBN_2_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
#                                            PctGrazing, GRBarnNRate,
#                                            Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Melt


class TestMelt(VariableUnitTest):

    def test_Melt(self):
        z = self.z
        test = Melt.Melt(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec)
        test1 = Melt.Melt_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec)
        np.testing.assert_array_almost_equal(
            Melt.Melt(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec),
            Melt.Melt_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec), decimal=7)



from numpy import maximum
from numpy import minimum
from numpy import reshape
from numpy import resize
from numpy import zeros

from gwlfe.AFOS.GrazingAnimals.Loads.GRInitBarnN import GRInitBarnN
from gwlfe.AFOS.GrazingAnimals.Loads.GRInitBarnN import GRInitBarnN_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.LossFactAdj import LossFactAdj
from gwlfe.MultiUse_Fxns.LossFactAdj import LossFactAdj_f


def GRLostBarnN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN):
    result = zeros((NYrs, 12))
    loss_fact_adj = LossFactAdj(NYrs, Prec, DaysMonth)
    gr_init_barn_n = GRInitBarnN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
                                 PctGrazing)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = (gr_init_barn_n[i] * GRBarnNRate[i] * loss_fact_adj[Y][i]
                            - gr_init_barn_n[i] * GRBarnNRate[i] * loss_fact_adj[Y][i] * AWMSGrPct * GrAWMSCoeffN
                            + gr_init_barn_n[i] * GRBarnNRate[i] * loss_fact_adj[Y][i] * RunContPct * RunConCoeffN)
            if result[Y][i] > gr_init_barn_n[i]:
                result[Y][i] = gr_init_barn_n[i]
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


@memoize
def GRLostBarnN_f(NYrs, Prec, DaysMonth, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
                  PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN):
    loss_fact_adj = LossFactAdj_f(Prec, DaysMonth)
    gr_init_barn_n = GRInitBarnN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing)

    years_gr_init_barn_gr_barn_n_rate = resize(gr_init_barn_n * GRBarnNRate,
                                               (NYrs, 12)) * loss_fact_adj  # TODO: what is a better name for this
    result = years_gr_init_barn_gr_barn_n_rate + \
             years_gr_init_barn_gr_barn_n_rate * (AWMSGrPct * GrAWMSCoeffN - RunContPct * RunConCoeffN)
    result = minimum(result, resize(gr_init_barn_n, (NYrs, 12)))
    result = maximum(result, 0)
    return reshape(result, (NYrs, 12))



from numpy import repeat
from numpy import tile
from numpy import where
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water, Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.Retention import Retention, Retention_f


@memoize
def Qrun(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    retention = Retention(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        result[Y][i][j][l] = 0
                        if CN[l] > 0:
                            if water[Y][i][j] >= 0.2 * retention[Y][i][j][l]:
                                result[Y][i][j][l] = (water[Y][i][j] - 0.2 * retention[Y][i][j][l]) ** 2 / (
                                        water[Y][i][j] + 0.8 * retention[Y][i][j][l])
    return result


@memoize
def Qrun_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu))
    water = repeat(Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)[:, :, :, None], nlu, axis=3)
    TempE = repeat(Temp[:, :, :, None], nlu, axis=3)
    cnrur = tile(CN[None, None, None, :], (NYrs, 12, 31, 1))
    retention = Retention_f(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0)
    retention02 = 0.2 * retention
    nonzero = where((TempE > 0) & (water > 0.01) & (water >= retention02) & (cnrur > 0))
    result[nonzero] = (water[nonzero] - retention02[nonzero]) ** 2 / (water[nonzero] + 0.8 * retention[nonzero])
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import CNumImpervReten


class TestCNumImpervReten(VariableUnitTest):

    def test_CNumImpervReten(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            CNumImpervReten.CNumImpervReten_f(z.NYrs, z.DaysMonth, z.Temp, z.Prec, z.InitSnow_0, z.AntMoist_0, z.NRur,
                                              z.NUrb, z.CNI_0, z.Grow_0),
            CNumImpervReten.CNumImpervReten(z.NYrs, z.DaysMonth, z.Temp, z.Prec, z.InitSnow_0, z.AntMoist_0, z.NRur,
                                            z.NUrb, z.CNI_0, z.Grow_0), decimal=7)



from numpy import repeat
from numpy import reshape
from numpy import sum
from numpy import where
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.Qrun import Qrun
from gwlfe.MultiUse_Fxns.Runoff.Qrun import Qrun_f
from gwlfe.MultiUse_Fxns.Runoff.Retention import Retention
from gwlfe.MultiUse_Fxns.Runoff.Retention import Retention_f


@memoize
def RurQRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0):
    result = zeros((NYrs, 16, 12))
    nlu = NLU(NRur, NUrb)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    retention = Retention(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0)
    qrun = Qrun(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0)
    for Y in range(NYrs):
        for i in range(12):
            for l in range(nlu):
                result[Y, l, i] = 0.0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        if CN[l] > 0:
                            if water[Y][i][j] >= 0.2 * retention[Y][i][j][l]:
                                result[Y][l][i] += qrun[Y][i][j][l]
                            else:
                                pass
                        else:
                            pass
                else:
                    pass

    return result


@memoize
def RurQRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0):
    water = reshape(repeat(Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec), repeats=NRur, axis=2),
                    (NYrs, 12, 31, NRur))
    retention = Retention_f(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0)[:, :, :, :NRur]
    qrun = Qrun_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0)[:, :, :, :NRur]
    return sum(where((water >= 0.2 * retention) & (CN[:NRur] > 0), qrun, 0), axis=2)



from numpy import sum
from numpy import zeros

# from Timer import time_function
from gwlfe.Input.WaterBudget.GrFlow import GrFlow
from gwlfe.Input.WaterBudget.GrFlow import GrFlow_f
from gwlfe.Memoization import memoize


@memoize
def GroundWatLE(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    result = zeros((NYrs, 12))
    grflow = GrFlow(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i] = result[Y][i] + grflow[Y][i][j]
    return result


@memoize
def GroundWatLE_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    grflow = GrFlow_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                      Imper,
                      ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef)
    return sum(grflow, axis=2)

# THIS IS A FUNCTION THAT FULLY SATISFIES GroundWatLE and matches the original model output. NOTHING IS SEPARATED YET
# def GroundWatLE_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
#            ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef, Landuse, TileDrainDensity):
#     result = np.zeros((NYrs, 12))
#     areatotal = AreaTotal(NRur, NUrb, Area) #4129.0
#     agareatotal = AgAreaTotal(NRur, Landuse, Area) # 2499.0
#     grflow = GrFlow(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
#            ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef)
#     gwagle = np.zeros((NYrs, 12))
#     tiledraingw = np.zeros((NYrs, 12))
#     for Y in range(NYrs):
#         for i in range(12):
#             for j in range(DaysMonth[Y][i]):
#                 result[Y][i] = result[Y][i] + grflow[Y][i][j]
#             if areatotal > 0:
#                 gwagle[Y][i] = (gwagle[Y][i] + (result[Y][i] * (agareatotal / areatotal)))
#             tiledraingw[Y][i] = (tiledraingw[Y][i] + [gwagle[Y][i] * TileDrainDensity])
#             result[Y][i] = result[Y][i] - tiledraingw[Y][i]
#             if result[Y][i] < 0:
#                 result[Y][i] = 0
#     #return result
#     pass



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Loads import NGLoadN
from gwlfe.enums import YesOrNo


class TestNGLoadN(VariableUnitTest):
    def test_NGLoadN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGLoadN.NGLoadN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            NGLoadN.NGLoadN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN)[
                z.GrazingAnimal_0 == YesOrNo.NO], decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.LandUse.Ag import AGSTRM


class TestAGSTRM(VariableUnitTest):

    def test_AGSTRM(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AGSTRM.AGSTRM_f(z.AgLength, z.StreamLength),
            AGSTRM.AGSTRM(z.AgLength, z.StreamLength), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns import PtSrcFlow


class TestPrecipitation(VariableUnitTest):

    def test_PtSrcFlow(self):
        z = self.z
        np.testing.assert_array_almost_equal(PtSrcFlow.PtSrcFlow_f(z.NYrs, z.PointFlow),
                                             PtSrcFlow.PtSrcFlow(z.NYrs, z.PointFlow), decimal=7)

    def test_AvPtSrcFlow(self):
        z = self.z
        z.PtSrcFlow = PtSrcFlow.PtSrcFlow_f(z.NYrs, z.PointFlow)
        np.testing.assert_array_almost_equal(PtSrcFlow.AvPtSrcFlow_f(z.PointFlow),
                                             PtSrcFlow.AvPtSrcFlow(z.NYrs, z.PtSrcFlow), decimal=7)



from numpy import sum

from gwlfe.Input.LandUse.RurAreaTotal import RurAreaTotal
from gwlfe.Input.LandUse.RurAreaTotal import RurAreaTotal_f
from gwlfe.Memoization import memoize


@memoize
def AvCNRur(NRur, Area, CN):
    result = 0
    rurareatotal = RurAreaTotal(NRur, Area)
    # Get the area weighted average CN for rural areas
    for l in range(NRur):
        # Calculate average area weighted CN and KF
        result += (CN[l] * Area[l] / rurareatotal) if rurareatotal > 0 else 0
    return result


def AvCNRur_f(NRur, Area, CN):
    rurareatotal = RurAreaTotal_f(NRur, Area)
    if (rurareatotal > 0):
        return sum(CN * Area / rurareatotal)
    else:
        return 0



from numpy import sum

from .NGLostBarnN import NGLostBarnN


def NGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate, Prec,
                   DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN):
    # result = zeros((NYrs,))
    result = sum(
        NGLostBarnN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate, Prec,
                    DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN), axis=1)
    return result


def NGLostBarnNSum_f():
    pass



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.Animals import TotPAEU


class TestTotPAEU(VariableUnitTest):

    def test_TotPAEU(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            TotPAEU.TotPAEU_f(z.NumAnimals, z.AvgAnimalWt),
            TotPAEU.TotPAEU(z.NumAnimals, z.AvgAnimalWt), decimal=7)



from numpy import maximum
from numpy import zeros

from gwlfe.Input.LandUse.Ag.TileDrainGW import TileDrainGW
from gwlfe.Input.LandUse.Ag.TileDrainGW import TileDrainGW_f
from gwlfe.Input.WaterBudget.GroundWatLE import GroundWatLE
from gwlfe.Input.WaterBudget.GroundWatLE import GroundWatLE_f
from gwlfe.Memoization import memoize


@memoize
def GroundWatLE_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                  Landuse, TileDrainDensity):
    result = zeros((NYrs, 12))
    tiledraingw = TileDrainGW(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                              RecessionCoef,
                              SeepCoef, Landuse, TileDrainDensity)
    grounwatle_1 = GroundWatLE(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                               CNP_0, Imper,
                               ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                               SeepCoef)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = grounwatle_1[Y][i] - tiledraingw[Y][i]
        if result[Y][i] < 0:
            result[Y][i] = 0
    return result


@memoize
def GroundWatLE_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                    Landuse, TileDrainDensity):
    result = zeros((NYrs, 12))
    tiledraingw = TileDrainGW_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0,
                                Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                RecessionCoef,
                                SeepCoef, Landuse, TileDrainDensity)
    grounwatle = GroundWatLE_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                               Grow_0,
                               CNP_0, Imper,
                               ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                               RecessionCoef,
                               SeepCoef)
    result = grounwatle - tiledraingw
    return maximum(result, 0)



from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.AgRunoff import AgRunoff
from gwlfe.MultiUse_Fxns.Runoff.AgRunoff import AgRunoff_f


@memoize
def TileDrainRO(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse, Area,
                TileDrainDensity):
    result = zeros((NYrs, 12))
    ag_runoff = AgRunoff(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse, Area)
    for Y in range(NYrs):
        for i in range(12):
            # CALCULATE THE SURFACE RUNOFF PORTION OF TILE DRAINAGE
            result[Y][i] = ag_runoff[Y][i] * TileDrainDensity
    return result


@memoize
def TileDrainRO_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse, Area,
                  TileDrainDensity):
    return AgRunoff_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse,
                      Area) * TileDrainDensity



from numpy import sum
from numpy import zeros

from gwlfe.AFOS.GrazingAnimals.Loads.GrazingN import GrazingN
from gwlfe.AFOS.GrazingAnimals.Loads.GrazingN import GrazingN_f
from gwlfe.Memoization import memoize


def GRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = zeros((12,))
    grazing_n = GrazingN(PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        result[i] = PctStreams[i] * grazing_n[i]
    return result


@memoize
def GRStreamN_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    return PctStreams * GrazingN_f(PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)


@memoize
def AvGRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = 0
    gr_stream_n = GRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        result += gr_stream_n[i]
    return result


@memoize
def AvGRStreamN_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    return sum(GRStreamN_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN))



from numpy import hstack
from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.ErosWashoff import ErosWashoff
from gwlfe.MultiUse_Fxns.Erosion.ErosWashoff import ErosWashoff_f
from gwlfe.MultiUse_Fxns.Erosion.SedDelivRatio import SedDelivRatio
from gwlfe.MultiUse_Fxns.Runoff.pRunoff import pRunoff
from gwlfe.MultiUse_Fxns.Runoff.pRunoff import pRunoff_f
from gwlfe.Output.Loading.LuLoad import LuLoad
from gwlfe.Output.Loading.LuLoad import LuLoad_f


@memoize
def LuTotPhos(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, PhosConc, ManPhos,
              ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2, LastManureMonth2, SedDelivRatio_0, KF,
              LS, C, P, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm,
              UrbBMPRed, FilterWidth, PctStrmBuf, Acoef, SedPhos, CNI_0):
    result = zeros((NYrs, 16))
    p_runoff = pRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, PhosConc,
                       ManuredAreas, FirstManureMonth, LastManureMonth, ManPhos, FirstManureMonth2,
                       LastManureMonth2)
    sed_deliv_ratio = SedDelivRatio(SedDelivRatio_0)
    eros_washoff = ErosWashoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Acoef,
                               KF, LS, C, P, Area)
    lu_load = LuLoad(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                     AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual,
                     LoadRateImp, LoadRatePerv, Storm, UrbBMPRed, FilterWidth, PctStrmBuf)
    nlu = NLU(NRur, NUrb)
    for Y in range(NYrs):
        for i in range(12):
            # Add in the urban calucation for sediment
            for l in range(NRur):
                result[Y][l] += p_runoff[Y][i][l]
                result[Y][l] += 0.001 * sed_deliv_ratio * eros_washoff[Y][l][i] * SedPhos

            for l in range(NRur, nlu):
                result[Y][l] += lu_load[Y][l][1] / NYrs / 2
    return result


@memoize
def LuTotPhos_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, PhosConc, ManPhos,
                ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2, LastManureMonth2, SedDelivRatio_0,
                KF,
                LS, C, P, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm,
                UrbBMPRed, FilterWidth, PctStrmBuf, Acoef, SedPhos, CNI_0):
    p_runoff = sum(
        pRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, PhosConc,
                  ManuredAreas, FirstManureMonth, LastManureMonth, ManPhos, FirstManureMonth2,
                  LastManureMonth2), axis=1)
    sed_deliv_ratio = SedDelivRatio(SedDelivRatio_0)

    eros_washoff = sum(ErosWashoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, Acoef,
                                     KF, LS, C, P, Area), axis=1)

    lu_load = LuLoad_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                       Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                       LoadRatePerv, Storm, UrbBMPRed, FilterWidth, PctStrmBuf)[:, :, 1]
    # luLoad is not needed because it is only defined for NUrb land use, and the others are only defined for NRur
    return hstack(
        (p_runoff + 0.001 * sed_deliv_ratio * eros_washoff * SedPhos, 12. * lu_load / NYrs / 2))  # + lu_load / NYrs / 2



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Loads import NGInitBarnN


class TestNGInitBarnN(VariableUnitTest):
    def test_NGInitBarnN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGInitBarnN.NGInitBarnN_f(z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            NGInitBarnN.NGInitBarnN(z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN)[None,
            :], decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import RuralQTotal


class TestRuralQTotal(VariableUnitTest):

    def test_RuralQTotal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            RuralQTotal.RuralQTotal_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.CN, z.NUrb,
                                      z.AntMoist_0, z.Grow_0, z.Area),
            RuralQTotal.RuralQTotal(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.CN, z.NUrb,
                                    z.AntMoist_0, z.Grow_0, z.Area), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Loads import InitNgN


class TestInitNgN(VariableUnitTest):
    def test_InitNgN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            InitNgN.InitNgN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            InitNgN.InitNgN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Withdrawal


class TestWithdrawal(VariableUnitTest):
    def test_Withdrawal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Withdrawal.Withdrawal_f(z.NYrs, z.StreamWithdrawal, z.GroundWithdrawal),
            Withdrawal.Withdrawal(z.NYrs, z.StreamWithdrawal, z.GroundWithdrawal), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import CNP


class TestCNP(VariableUnitTest):

    def test_CNP(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            CNP.CNP_f(z.NRur, z.NUrb, z.CNP_0),
            CNP.CNP(z.NRur, z.NUrb, z.CNP_0), decimal=7)



from numba.pycc import CC
from numpy import zeros

cc = CC('UnsatStor_inner_compiled')


@cc.export('UnsatStor_inner', '(int64,int64[:,::1],float64,float64,float64[:,:,::1],float64[:,:,::1])')
def UnsatStor_inner(NYrs, DaysMonth, MaxWaterCap, UnsatStor_0, infiltration, DailyET):
    unsatstor = zeros((NYrs, 12, 31))
    unsatstor_carryover = UnsatStor_0
    et = zeros((NYrs, 12, 31))
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                unsatstor[Y][i][j] = unsatstor_carryover
                unsatstor[Y][i][j] = unsatstor[Y][i][j] + infiltration[Y][i][j]
                if DailyET[Y][i][j] >= unsatstor[Y][i][j]:
                    et[Y][i][j] = unsatstor[Y][i][j]
                    unsatstor[Y][i][j] = 0
                else:
                    et[Y][i][j] = DailyET[Y][i][j]
                    unsatstor[Y][i][j] = unsatstor[Y][i][j] - DailyET[Y][i][j]
                if unsatstor[Y][i][j] > MaxWaterCap:
                    unsatstor[Y][i][j] = MaxWaterCap
                else:
                    pass
                unsatstor_carryover = unsatstor[Y][i][j]
    return unsatstor, et, unsatstor_carryover



from gwlfe.Memoization import memoize


@memoize
def AGSTRM(AgLength, StreamLength):
    # OBTAIN THE LENGTH OF STREAMS IN AGRICULTURAL AREAS
    result = 0.0
    result = AgLength / StreamLength if StreamLength > 0 else 0
    return result


@memoize
def AGSTRM_f(AgLength, StreamLength):
    # OBTAIN THE LENGTH OF STREAMS IN AGRICULTURAL AREAS
    if (StreamLength > 0):
        return AgLength / StreamLength
    else:
        return 0



from numpy import where
from numpy import zeros

from gwlfe.Memoization import memoize


def Rain_inner(NYrs, DaysMonth, Temp, Prec):
    result = zeros((NYrs, 12, 31))
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = 0
                if Temp[Y][i][j] <= 0:
                    pass
                else:
                    result[Y][i][j] = Prec[Y][i][j]
    return result


@memoize
def Rain(NYrs, DaysMonth, Temp, Prec):
    return Rain_inner(NYrs, DaysMonth, Temp, Prec)


@memoize
def Rain_f(Temp, Prec):
    return where(Temp <= 0, 0, Prec)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
from calendar import monthrange
from decimal import Decimal

from numpy import zeros
from numpy import ndarray

class DataModel(object):
    def __init__(self, data=None):
        self.__dict__.update(self.defaults())
        self.__dict__.update(data or {})
        self.__dict__.update(self.date_guides())

    def defaults(self):
        NLU = 16
        NAnimals = 9

        return {
            'BasinId': 0,
            'AgSlope3to8': 0,
            'NumUAs': 0,
            'UABasinArea': 0,

            # Variables that are passed to PRedICT and are no longer used.
            'InName': '',
            'OutName': '',
            'UnitsFileFlag': 1,
            'AssessDate': '',
            'VersionNo': '',
            'ProjName': '',

            'NLU': NLU,
            'NAnimals': NAnimals,

            'Landuse': zeros(NLU, dtype=object),
            'Area': zeros(NLU),
            'CN': zeros(NLU),
            'KF': zeros(NLU),
            'LS': zeros(NLU),
            'C': zeros(NLU),
            'P': zeros(NLU),

            'NumNormalSys': zeros(12, dtype=int),
            'NumPondSys': zeros(12),
            'NumShortSys': zeros(12),
            'NumDischargeSys': zeros(12),
            'NumSewerSys': zeros(12),

            'SEDFEN': 0,
            'NFEN': 0,
            'PFEN': 0,

            'n86': 0,
            'n87': 0,
            'n88': 0,
            'n89': 0,
            'n90': 0,
            'n91': 0,
            'n92': 0,
            'n93': 0,
            'n94': 0,
            'n95': 0,
            'n95b': 0,
            'n95c': 0,
            'n95d': 0,
            'n95e': 0,

            'n96': 0,
            'n97': 0,
            'n98': 0,
            'n99': 0,
            'n99b': 0,
            'n99c': 0,
            'n99d': 0,
            'n99e': 0,
            'n100': 0,
            'n101': 0,
            'n101b': 0,
            'n101c': 0,
            'n101d': 0,
            'n101e': 0,
            'n102': 0,
            'n103a': 0,
            'n103b': 0,
            'n103c': 0,
            'n103d': 0,

            'n104': 0,
            'n105': 0,
            'n106': 0,
            'n106b': 0,
            'n106c': 0,
            'n106d': 0,
            'n107': 0,
            'n107b': 0,
            'n107c': 0,
            'n107d': 0,
            'n107e': 0,

            'Storm': 0,
            'CSNAreaSim': 0,
            'CSNDevType': 'None'
        }

    def date_guides(self):
        model = self.__dict__
        output = {}
        if 'WxYrBeg' in model and 'WxYrEnd' in model:
            begyear = model['WxYrBeg']
            endyear = model['WxYrEnd']
            year_range = endyear - begyear + 1
            month_abbr = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
                          'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']

            if 'DaysMonth' not in model:
                output['DaysMonth'] = zeros((year_range, 12),
                                            dtype=int)
                for y in range(year_range):
                    year = begyear + y
                    for m in range(12):
                        output['DaysMonth'][y][m] = monthrange(year, m + 1)[1]

            if 'WxMonth' not in model:
                output['WxMonth'] = [month_abbr
                                     for y in range(year_range)]

            if 'WxYear' not in model:
                output['WxYear'] = [[begyear + y] * 12
                                    for y in range(year_range)]
        return output

    def __str__(self):
        return '<GWLF-E DataModel>'

    def tojson(self):
        return json.dumps(self.__dict__, cls=NumpyAwareJSONEncoder)


class NumpyAwareJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, ndarray):
            return obj.tolist()
        elif isinstance(obj, Decimal):
            return float(obj)
        return json.JSONEncoder.default(self, obj)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.LandUse import AreaTotal


class TestAreaTotal(VariableUnitTest):

    def test_AreaTotal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AreaTotal.AreaTotal_f(z.Area),
            AreaTotal.AreaTotal(z.NRur, z.NUrb, z.Area), decimal=7)



from numba.pycc import CC
from numpy import zeros

cc = CC('CNumImperv_inner_compiled')


@cc.export('CNumImperv_inner',
           '(int64, int64, int64[:,::1], float64[:,:,::1], int64, float64[:,::1], float64[:,:,::1], float64[:,:,::1], boolean[::1], float64[:,:,::1])')
def CNumImperv_inner(NYrs, NRur, DaysMonth, Temp, nlu, cni, water, melt, grow_factor, amc5):
    result = zeros((NYrs, 12, 31, nlu))
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        pass
                    else:
                        for l in range(NRur, nlu):
                            if cni[1][l] > 0:
                                if melt[Y][i][j] <= 0:
                                    if grow_factor[i] > 0:
                                        # Growing season
                                        if amc5[Y][i][j] >= 5.33:
                                            result[Y][i][j][l] = cni[2][l]
                                        elif amc5[Y][i][j] < 3.56:
                                            result[Y][i][j][l] = cni[0][l] + (
                                                    cni[1][l] - cni[0][l]) * amc5[Y][i][j] / 3.56
                                        else:
                                            result[Y][i][j][l] = cni[1][l] + (cni[2][l] - cni[1][l]) * (
                                                    amc5[Y][i][j] - 3.56) / 1.77
                                    else:
                                        # Dormant season
                                        if amc5[Y][i][j] >= 2.79:
                                            result[Y][i][j][l] = cni[2][l]
                                        elif amc5[Y][i][j] < 1.27:
                                            result[Y][i][j][l] = cni[0][l] + (
                                                    cni[1][l] - cni[0][l]) * amc5[Y][i][j] / 1.27
                                        else:
                                            result[Y][i][j][l] = cni[1][l] + (cni[2][l] - cni[1][l]) * (
                                                    amc5[Y][i][j] - 1.27) / 1.52
                                else:
                                    result[Y][i][j][l] = cni[2][l]
    return result



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Imported from PrelimCalculations.bas
"""

import logging
from .enums import LandUse

log = logging.getLogger(__name__)


def InitialCalculations(z):
    # Obtain areas in Ha for Urban, Agricultural and Forested landuse
    for l in range(z.NRur):
        if z.Landuse[l] is LandUse.FOREST:
            z.ForestAreaTotal += z.Area[l]

    # ANTECEDANT MOISTURE OUT TO 5 DAYS
    z.AMC5_f = 0
    for k in range(5):
        z.AMC5_f += z.AntMoist[k]



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import RurEros


class TestRurEros(VariableUnitTest):

    def test_elementwise_RurEros(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/RurEros.npy"),
            RurEros.RurEros_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C, z.P,
                              z.Area), decimal=7)

    def test_RurEros(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            RurEros.RurEros_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C, z.P,
                              z.Area),
            RurEros.RurEros(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C, z.P,
                            z.Area), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import StreamBankEros


class TestStreamBankEros(VariableUnitTest):

    def test_StreamBankEros(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            StreamBankEros.StreamBankEros_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                            z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                            z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                            z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b,
                                            z.Landuse, z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal,
                                            z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                            z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength),
            StreamBankEros.StreamBankEros(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                          z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                          z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                          z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                          z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal, z.GroundWithdrawal,
                                          z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                                          z.AvSlope, z.SedAAdjust, z.StreamLength), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.LandUse.Urb import UrbAreaTotal


class TestUrbAreaTotal(VariableUnitTest):

    def test_UrbAreaTotal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            UrbAreaTotal.UrbAreaTotal_f(z.NRur, z.NUrb, z.Area),
            UrbAreaTotal.UrbAreaTotal(z.NRur, z.NUrb, z.Area), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import CNI


class TestCNI(VariableUnitTest):

    def test_CNI(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            CNI.CNI_f(z.NRur, z.NUrb, z.CNI_0),
            CNI.CNI(z.NRur, z.NUrb, z.CNI_0), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import Qrun


class TestQrun(VariableUnitTest):
    def test_Qrun(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Qrun.Qrun_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.CN, z.AntMoist_0,
                        z.Grow_0),
            Qrun.Qrun(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.CN, z.AntMoist_0, z.Grow_0),
            decimal=7)



from gwlfe.Input.LandUse.AreaTotal import AreaTotal
from gwlfe.Memoization import memoize


@memoize
def TotAreaMeters(NRur, NUrb, Area):
    result = 0.0
    areatotal = AreaTotal(NRur, NUrb, Area)
    result = areatotal * 10000
    return result



from numpy import sum
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.RurEros import RurEros
from gwlfe.MultiUse_Fxns.Erosion.RurEros import RurEros_f


@memoize
def Erosion(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area):
    result = zeros((NYrs, 12))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    rureros = RurEros(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                for l in range(NRur):
                    if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                        result[Y][i] = result[Y][i] + rureros[Y][i][j][l]
                    else:
                        pass
    return result


@memoize
def Erosion_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area):
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    rureros = RurEros_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    return sum(sum(rureros, axis=3), axis=2)



import os


variable_name = input("Enter the name of the variable:")

#write variable file
var_filename = "%s.py"%(variable_name)
if os.path.exists("gwlfe/"+var_filename):
    raise IOError("File already exists")
else:
    with open("gwlfe/"+var_filename,"w") as file:
        file.write("from numpy import zeros\n"
                    "from Memoization import memoize\n\n\n"
                    "def %s():\n"
                    "    pass\n"
                    "\n\n"
                    "def %s_f():\n"
                    "    pass\n"%(variable_name,variable_name)
                   )

#write test file
test_filename = "test_%s.py"%(variable_name)
if os.path.exists("test/"+test_filename):
    raise IOError("File already exists")
else:
    with open("test/"+test_filename,"w") as file:
        file.write("import numpy as np\n"
                    "from .VariableUnitTest import VariableUnitTest\n"
                    "from unittest import skip\n"
                    "from gwlfe import {variable}\n"
                    "\n\n"
                    "class Test{variable}(VariableUnitTest):\n"
                    "    @skip('Not Ready Yet.')\n"
                    "    def test_{variable}(self):\n"
                    "        z = self.z\n"
                    "        np.testing.assert_array_almost_equal(\n"
                    "            {variable}.{variable}_f(),\n"
                    "            {variable}.{variable}(), decimal=7)".format(variable=variable_name))


from numpy import hstack
from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.RurQRunoff import RurQRunoff
from gwlfe.MultiUse_Fxns.Runoff.RurQRunoff import RurQRunoff_f
from gwlfe.MultiUse_Fxns.Runoff.UrbQRunoff import UrbQRunoff
from gwlfe.MultiUse_Fxns.Runoff.UrbQRunoff import UrbQRunoff_f


def LuRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, CNI_0, CNP_0,
             AntMoist_0, Grow_0, Imper, ISRR, ISRA, CN):
    result = zeros((NYrs, 16))
    nlu = NLU(NRur, NUrb)
    urb_q_runoff = UrbQRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, CNI_0, CNP_0,
                              AntMoist_0, Grow_0, Imper, ISRR, ISRA)
    rur_q_runoff = RurQRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0)
    for Y in range(NYrs):
        for i in range(12):
            # Calculate landuse runoff for rural areas
            for l in range(NRur):
                result[Y][l] += rur_q_runoff[Y][l][i]
        for i in range(12):
            # Calculate landuse runoff for urban areas
            for l in range(NRur, nlu):
                result[Y][l] += urb_q_runoff[Y][l][i]
    return result


@memoize
def LuRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, CNI_0, CNP_0,
               AntMoist_0, Grow_0, Imper, ISRR, ISRA, CN):
    return hstack(
        (sum(RurQRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0), axis=1),
         sum(UrbQRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, CNI_0, CNP_0,
                          AntMoist_0, Grow_0, Imper, ISRR, ISRA), axis=1),
         ))



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Initialize variables and perfom some preliminary calculations.

Imported from ReadAllDataFiles.bas
"""

import logging
from .Input.WaterBudget.FlowDays import FlowDays
from .enums import SweepType, YesOrNo
from . import PrelimQualCalculations

log = logging.getLogger(__name__)

from .AFOS.GrazingAnimals.Loads.GrazingN import GrazingN_f
from .AFOS.GrazingAnimals.Losses.GRStreamN import GRStreamN_f
from .AFOS.GrazingAnimals.Loads.GRAccManAppN import GRAccManAppN_f
from .AFOS.nonGrazingAnimals.Loads.NGAppManN import NGAppManN_f
from .AFOS.nonGrazingAnimals.Losses.NGLostBarnN import NGLostBarnN_f
from .AFOS.GrazingAnimals.Losses.GRLostBarnN import GRLostBarnN_f
from .AFOS.GrazingAnimals.Losses.GRLostManN import GRLostManN_f
from .AFOS.nonGrazingAnimals.Losses.NGLostManN import NGLostManN_f
from .AFOS.GrazingAnimals.Losses.GRLossN import GRLossN_f
from .Input.Animals.GrazingAnimal import GrazingAnimal


def ReadAllData(z):
    z.GrazingN = GrazingN_f(z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN)
    z.GRStreamN = GRStreamN_f(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                              z.AnimalDailyN)
    z.GRAccManAppN = GRAccManAppN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.GRPctManApp,
                                    z.PctGrazing)
    z.NGAppManN = NGAppManN_f(z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN)
    z.NGLostBarnN = NGLostBarnN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                  z.NGBarnNRate, z.Prec,
                                  z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN)

    z.GRLostBarnN = GRLostBarnN_f(z.NYrs, z.Prec, z.DaysMonth, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                  z.AnimalDailyN, z.GRPctManApp, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct,
                                  z.GrAWMSCoeffN, z.RunContPct, z.RunConCoeffN)

    z.GRLostManN = GRLostManN_f(z.NYrs, z.GRPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                z.GRAppNRate,
                                z.Prec, z.DaysMonth, z.GRPctSoilIncRate)

    z.NGLostManN = NGLostManN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                z.NGAppNRate,
                                z.Prec, z.DaysMonth, z.NGPctSoilIncRate)

    z.GRLossN = GRLossN_f(z.NYrs, z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                          z.AnimalDailyN,
                          z.GrazingNRate, z.Prec, z.DaysMonth)

    # If RunQual output is requested, then redim RunQual values
    PrelimQualCalculations.ReDimRunQualVars()

    # Set the Total AEU to the value from the Animal Density layer

    for i in range(12):
        z.AnnDayHrs += z.DayHrs[i]

    if z.SepticFlag is YesOrNo.YES:
        for i in range(12):
            z.SepticsDay[i] = (z.NumNormalSys[i] + z.NumPondSys[i] +
                               z.NumShortSys[i] + z.NumDischargeSys[i])

    for i in range(12):
        if z.SweepType is SweepType.VACUUM:
            if z.StreetSweepNo[i] == 0:
                z.SweepFrac[i] = 1
            if z.StreetSweepNo[i] == 1:
                z.SweepFrac[i] = 0.94
            if z.StreetSweepNo[i] == 2:
                z.SweepFrac[i] = 0.89
            if z.StreetSweepNo[i] == 3:
                z.SweepFrac[i] = 0.84
            if z.StreetSweepNo[i] >= 4:
                z.SweepFrac[i] = 0.79
        elif z.SweepType is SweepType.MECHANICAL:
            if z.StreetSweepNo[i] == 0:
                z.SweepFrac[i] = 1
            if z.StreetSweepNo[i] == 1:
                z.SweepFrac[i] = 0.99
            if z.StreetSweepNo[i] == 2:
                z.SweepFrac[i] = 0.98
            if z.StreetSweepNo[i] == 3:
                z.SweepFrac[i] = 0.97
            if z.StreetSweepNo[i] >= 4:
                z.SweepFrac[i] = 0.96
        else:
            raise ValueError('Invalid value for SweepType')

    # Get the Animal values
    z.InitGrP = 0
    z.InitGrFC = 0
    z.InitNgP = 0
    z.InitNgFC = 0

    for a in range(9):
        if GrazingAnimal(z.GrazingAnimal_0)[a] is YesOrNo.NO:
            z.NGLoadP[a] = (z.NumAnimals[a] * z.AvgAnimalWt[a] / 1000) * z.AnimalDailyP[a] * 365
            z.NGLoadFC[a] = (z.NumAnimals[a] * z.AvgAnimalWt[a] / 1000) * z.FCOrgsPerDay[a] * 365
            z.InitNgP += z.NGLoadP[a]
            z.InitNgFC += z.NGLoadFC[a]
        elif GrazingAnimal(z.GrazingAnimal_0)[a] is YesOrNo.YES:
            z.GRLoadP[a] = (z.NumAnimals[a] * z.AvgAnimalWt[a] / 1000) * z.AnimalDailyP[a] * 365
            z.GRLoadFC[a] = (z.NumAnimals[a] * z.AvgAnimalWt[a] / 1000) * z.FCOrgsPerDay[a] * 365
            z.InitGrP += z.GRLoadP[a]
            z.InitGrFC += z.GRLoadFC[a]
        else:
            raise ValueError('Unexpected value for GrazingAnimal')

    # Get the Non-Grazing Animal Worksheet values
    for i in range(12):
        # For Non-Grazing

        z.NGAccManAppP[i] += (z.InitNgP / 12) - (z.NGPctManApp[i] * z.InitNgP)

        if z.NGAccManAppP[i] < 0:
            z.NGAccManAppP[i] = 0

        z.NGAccManAppFC[i] += (z.InitNgFC / 12) - (z.NGPctManApp[i] * z.InitNgFC)

        if z.NGAccManAppFC[i] < 0:
            z.NGAccManAppFC[i] = 0

        z.NGAppManP[i] = z.NGPctManApp[i] * z.InitNgP
        z.NGInitBarnP[i] = z.NGAccManAppP[i] - z.NGAppManP[i]

        if z.NGInitBarnP[i] < 0:
            z.NGInitBarnP[i] = 0

        z.NGAppManFC[i] = z.NGPctManApp[i] * z.InitNgFC
        z.NGInitBarnFC[i] = z.NGAccManAppFC[i] - z.NGAppManFC[i]

        if z.NGInitBarnFC[i] < 0:
            z.NGInitBarnFC[i] = 0

    # Read the Grazing Animal Worksheet values

    for i in range(12):
        z.GrazingP[i] = z.PctGrazing[i] * (z.InitGrP / 12)
        z.GrazingFC[i] = z.PctGrazing[i] * (z.InitGrFC / 12)

        z.GRStreamP[i] = z.PctStreams[i] * z.GrazingP[i]
        z.GRStreamFC[i] = z.PctStreams[i] * z.GrazingFC[i]

        # Get the annual sum for FC
        z.AvGRStreamFC += z.GRStreamFC[i]
        z.AvGRStreamP += z.GRStreamP[i]

        z.GRAccManAppP[i] = (z.GRAccManAppP[i] + (z.InitGrP / 12)
                             - (z.GRPctManApp[i] * z.InitGrP) - z.GrazingP[i])
        if z.GRAccManAppP[i] < 0:
            z.GRAccManAppP[i] = 0

        z.GRAccManAppFC[i] = (z.GRAccManAppFC[i] + (z.InitGrFC / 12)
                              - (z.GRPctManApp[i] * z.InitGrFC) - z.GrazingFC[i])
        if z.GRAccManAppFC[i] < 0:
            z.GRAccManAppFC[i] = 0

        z.GRAppManP[i] = z.GRPctManApp[i] * z.InitGrP
        z.GRInitBarnP[i] = z.GRAccManAppP[i] - z.GRAppManP[i]
        if z.GRInitBarnP[i] < 0:
            z.GRInitBarnP[i] = 0

        z.GRAppManFC[i] = z.GRPctManApp[i] * z.InitGrFC
        z.GRInitBarnFC[i] = z.GRAccManAppFC[i] - z.GRAppManFC[i]
        if z.GRInitBarnFC[i] < 0:
            z.GRInitBarnFC[i] = 0

    z.AttenP = FlowDays(z.AttenFlowDist, z.AttenFlowVel) * z.AttenLossRateP
    z.AttenTSS = FlowDays(z.AttenFlowDist, z.AttenFlowVel) * z.AttenLossRateTSS
    z.AttenPath = FlowDays(z.AttenFlowDist, z.AttenFlowVel) * z.AttenLossRatePath

    # Calculate retention coefficients
    z.RetentFactorP = (1 - (z.ShedAreaDrainLake * z.RetentPLake))
    z.RetentFactorSed = (1 - (z.ShedAreaDrainLake * z.RetentSedLake))



from numpy import zeros

from .GRAppManN import GRAppManN
from .GRAppManN import GRAppManN_f
from gwlfe.AFOS.GrazingAnimals.Loads.GRAccManAppN import GRAccManAppN
from gwlfe.AFOS.GrazingAnimals.Loads.GRAccManAppN import GRAccManAppN_f


def GRInitBarnN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing):
    result = zeros((12,))
    gr_acc_man_app_n = GRAccManAppN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing)
    gr_app_man_n = GRAppManN(GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        result[i] = gr_acc_man_app_n[i] - gr_app_man_n[i]
    return result


def GRInitBarnN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing):
    return GRAccManAppN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
                          PctGrazing) - GRAppManN_f(GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import SedYield_1


class TestSedYield_f(VariableUnitTest):

    def test_SedYield_f(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            SedYield_1.SedYield_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                    z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
                                    z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                    z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow,
                                    z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt,
                                    z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength,
                                    z.n42b, z.n46c, z.n85d, z.AgLength, z.n42, z.n45, z.n85, z.UrbBankStab, z.Acoef,
                                    z.KF, z.LS, z.C, z.P, z.SedDelivRatio_0),
            SedYield_1.SedYield_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                  z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
                                  z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                  z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow,
                                  z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt,
                                  z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength,
                                  z.n42b, z.n46c, z.n85d, z.AgLength, z.n42, z.n45, z.n85, z.UrbBankStab, z.Acoef,
                                  z.KF, z.LS, z.C, z.P, z.SedDelivRatio_0), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Losses import NGLostBarnN


class TestNGLostBarnN(VariableUnitTest):

    def test_NGLostBarnN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGLostBarnN.NGLostBarnN(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                    z.AnimalDailyN, z.NGBarnNRate, z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN,
                                    z.RunContPct, z.RunConCoeffN),
            NGLostBarnN.NGLostBarnN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                      z.AnimalDailyN, z.NGBarnNRate, z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN,
                                      z.RunContPct, z.RunConCoeffN),
            decimal=7)

    def test_AvNGLostBarnN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGLostBarnN.AvNGLostBarnN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                        z.AnimalDailyN, z.NGBarnNRate,
                                        z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN),
            NGLostBarnN.AvNGLostBarnN(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                      z.AnimalDailyN, z.NGBarnNRate,
                                      z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN),
            decimal=7)

    def test_AvNGLostBarnNSum(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGLostBarnN.AvNGLostBarnNSum_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                           z.AnimalDailyN, z.NGBarnNRate, z.Prec, z.DaysMonth, z.AWMSNgPct,
                                           z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN),
            NGLostBarnN.AvNGLostBarnNSum(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                         z.AnimalDailyN, z.NGBarnNRate, z.Prec, z.DaysMonth, z.AWMSNgPct,
                                         z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Melt_1


# from gwlfe import MeltPest


class TestMelt_1(VariableUnitTest):

    def test_Melt_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Melt_1.Melt_1_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec),
            Melt_1.Melt_1(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec), decimal=7)



from .N7b import N7b
from .N7b import N7b_f
from gwlfe.Memoization import memoize


def AvAnimalNSum_1(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate,
                   GRAppNRate,
                   GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate,
                   Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp,
                   AWMSNgPct, NGBarnNRate, NgAWMSCoeffN, n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64):
    n7b = N7b(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
              GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
              GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN,
              n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64)
    result = n7b
    return result


@memoize
def AvAnimalNSum_1_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate,
                     GRAppNRate,
                     GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate,
                     Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp,
                     AWMSNgPct, NGBarnNRate, NgAWMSCoeffN, n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64):
    return N7b_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
                 GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
                 GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN,
                 n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import InitSnow


class TestInitSnow(VariableUnitTest):

    def test_InitSnow(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            InitSnow.InitSnow_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec),
            InitSnow.InitSnow(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec), decimal=7)



import hashlib

from numpy import ndarray


def memoize_with_args(f):
    """This memoization function checks to ensure that the called arguments are equal before returning memoized result.
    This is patched in for testing to ensure that there is less chance of unitentional coupling, but it comes at the
    expense of some performance"""

    class memodict():
        def __init__(self, f):
            self.f = f
            self.result = {}
            self.__name__ = f.__name__

        def __call__(self, *args):
            args_string = f.__name__
            for arg in args:
                if (isinstance(arg, ndarray)):
                    args_string += hashlib.sha1(arg).hexdigest() + ","
                else:
                    args_string += hashlib.sha1(str(arg)).hexdigest() + ","
            try:
                return self.result[args_string]
            except KeyError:
                self.result[args_string] = self.f(*args)
                return self.result[args_string]

    return memodict(f)


# def memoize(f): #use this function to disable memoization
#     return f

memoized_return_values = {}


def resetMemoization():
    global memoized_return_values
    memoized_return_values = {}


def memoize(f):
    """This memoization function does not check what arguments the function is called with. This requires that the model
    be reitinitalized between each run, but saves a significant amount of time on a single run"""

    class memodict(dict):
        def __init__(self, f):
            self.f = f
            self.__name__ = f.__name__

        def __call__(self, *args):
            global memoized_return_values
            try:
                return memoized_return_values[self.__name__]
            except KeyError:
                ret = memoized_return_values[self.__name__] = self.f(*args)
                return ret

    return memodict(f)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Losses import AvGRLostBarnN


class TestAvGRLostBarnN(VariableUnitTest):

    def test_AvGRLostBarnN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvGRLostBarnN.AvGRLostBarnN_f(z.NYrs, z.Prec, z.DaysMonth, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                          z.AnimalDailyN, z.GRPctManApp,
                                          z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct,
                                          z.RunConCoeffN),
            AvGRLostBarnN.AvGRLostBarnN(z.NYrs, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                        z.GRPctManApp, z.PctGrazing, z.GRBarnNRate, z.Prec, z.DaysMonth, z.AWMSGrPct,
                                        z.GrAWMSCoeffN, z.RunContPct, z.RunConCoeffN), decimal=7)



from numpy import maximum
from numpy import repeat
from numpy import reshape
from numpy import where
from numpy import zeros

from gwlfe.BMPs.Stream.UrbLoadRed import UrbLoadRed
from gwlfe.BMPs.Stream.UrbLoadRed import UrbLoadRed_f
from gwlfe.Input.LandUse.LU_1 import LU_1
from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal import AdjUrbanQTotal
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal import AdjUrbanQTotal_f
from gwlfe.MultiUse_Fxns.Runoff.WashImperv import WashImperv
from gwlfe.MultiUse_Fxns.Runoff.WashImperv import WashImperv_f
from gwlfe.MultiUse_Fxns.Runoff.WashPerv import WashPerv
from gwlfe.MultiUse_Fxns.Runoff.WashPerv import WashPerv_f


@memoize
def SurfaceLoad(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed):
    result = zeros((NYrs, 12, 31, 16, Nqual))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    adjurbanqtotal = AdjUrbanQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0, CNP_0,
                                    Imper, ISRR, ISRA, Qretention, PctAreaInfil)
    nlu = NLU(NRur, NUrb)
    washimperv = WashImperv(NYrs, DaysMonth, InitSnow_0, Temp, Prec, CNI_0, AntMoist_0, Grow_0, NRur, NUrb)
    washperv = WashPerv(NYrs, DaysMonth, InitSnow_0, Temp, Prec, CNP_0, AntMoist_0, Grow_0, NRur, NUrb)
    lu_1 = LU_1(NRur, NUrb)
    urbloadred = UrbLoadRed(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                            Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, Storm, UrbBMPRed)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if adjurbanqtotal[Y][i][j] > 0.001:
                        for l in range(NRur, nlu):
                            for q in range(Nqual):
                                if Area[l] > 0:
                                    result[Y][i][j][l][q] = (((LoadRateImp[l][q] * washimperv[Y][i][j][l] * (
                                            (Imper[l] * (1 - ISRR[lu_1[l]]) * (1 - ISRA[lu_1[l]]))
                                            # * (SweepFrac[i] + (
                                            # (1 - SweepFrac[i]) * ((1 - UrbSweepFrac) * Area[l]) / Area[l]))
                                            * 1)  # TODO For some reason, this commented out code always needs to evaluate to 1 in order for the separation to occur
                                                               + LoadRatePerv[l][q] * washperv[Y][i][j][l] * (
                                                                       1 - (Imper[l] * (1 - ISRR[lu_1[l]]) * (
                                                                       1 - ISRA[lu_1[l]]))))
                                                              * Area[l]) - urbloadred[Y][i][j][l][q])
                                else:
                                    result[Y][i][j][l][q] = 0

                                if result[Y][i][j][l][q] < 0:
                                    result[Y][i][j][l][q] = 0
                                else:
                                    pass
                    else:
                        pass
                else:
                    pass
    return result


@memoize
def SurfaceLoad_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                  Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                  LoadRatePerv, Storm, UrbBMPRed):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu - NRur, Nqual))
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    adjurbanqtotal = AdjUrbanQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                      Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil)
    nonzeroday = where((Temp > 0) & (water > 0.01) & (adjurbanqtotal > 0.001))
    washimperv = reshape(
        repeat(
            WashImperv_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, CNI_0, AntMoist_0, Grow_0, NRur, NUrb)[:, :, :,
            NRur:],
            repeats=Nqual,
            axis=3), (NYrs, 12, 31, nlu - NRur, Nqual))
    washperv = reshape(
        repeat(
            WashPerv_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, CNP_0, AntMoist_0, Grow_0, NRur, NUrb)[:, :, :, NRur:],
            repeats=Nqual,
            axis=3), (NYrs, 12, 31, nlu - NRur, Nqual))
    urbloadred = UrbLoadRed_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, Storm, UrbBMPRed)[:, :, :, NRur:]

    temp = reshape(repeat(Imper[NRur:] * (1 - ISRR) * (1 - ISRA), repeats=Nqual, axis=0), (-1, Nqual))
    # making an assumption that Area cannot be negative. Therefor where area = 0 result will be <= 0 and will be set to zero before returned (eliminating an if)
    result[nonzeroday] = (washimperv[nonzeroday] * LoadRateImp[NRur:] * temp +
                          washperv[nonzeroday] * LoadRatePerv[NRur:] * (1 - temp)) * reshape(
        repeat(Area[NRur:], repeats=Nqual, axis=0), (nlu - NRur, Nqual)) - urbloadred[nonzeroday]
    return maximum(result, 0)



from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Memoization import memoize


@memoize
def LU(NRur, NUrb):
    nlu = NLU(NRur, NUrb)
    result = zeros((nlu,)).astype("int")
    for l in range(NRur, nlu):
        result[l] = l - NRur
    return result

# @time_function
# lu is faster than lu_f
# def lu_f(NRur, NUrb):
#     nlu = NLU(NRur, NUrb)
#     result = np.zeros((nlu,)).astype("int")
#     result[NRur:nlu] = np.asarray(range(NRur, nlu)) - NRur
#     return result
